﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;

public class GameManager : MonoBehaviour
{
    public GameManager Instance { get; private set; }
    public GameObject GameManagerCamera;

    /// <summary>
    /// Awake is called when the script instance is being loaded.
    /// </summary>
    void Awake()
    {
        if (Instance != null)
        {
            DestroyImmediate(gameObject);
            return;
        }

        Instance = this;
        DontDestroyOnLoad(this.gameObject);
    }
    void Start()
    {
        LoadLevel("SampleScene");
    }

    private string _currentLevelName = string.Empty;
    public void LoadLevel(string levelName)
    {
        AsyncOperation asyncLoadingLevel = SceneManager.LoadSceneAsync(levelName, LoadSceneMode.Additive);
        if (asyncLoadingLevel == null)
            return;
        asyncLoadingLevel.completed += OnLoadOperationComplete;

        _currentLevelName = levelName;
    }

    public void UnloadLevel(string levelName)
    {
        AsyncOperation asyncUnloadingLevel = SceneManager.UnloadSceneAsync(levelName);
        if (asyncUnloadingLevel == null)
            return;
        asyncUnloadingLevel.completed += OnUnloadOperationComplete;

        _currentLevelName = levelName;
    }

    private void OnUnloadOperationComplete(AsyncOperation asyncOperation)
    {
        Debug.Log("Scene unload complete.");
    }

    private void OnLoadOperationComplete(AsyncOperation asyncOperation)
    {
        GameManagerCamera.SetActive(false);
        Debug.Log("Scene load complete.");
    }
}
