﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using TMPro;
using System;
using UnityEngine.SceneManagement;

public class Player : MonoBehaviour
{
    SpriteRenderer spriteRenderer;
    public Sprite sprNeutral;
    public Sprite sprLifting;
    GameObject holding = null;

    GameObject OnTopOf;
    IPickupable OnTopOfIPickable;

    public Transform dropPosition;

    public Gate.GateInfo onTopOfGate;
    public CoffeeMakerSwitch.CoffeeMakerSwitchInfo coffeeSwitch;

    public AudioClip sfxDrop;
    public AudioClip sfxPickup;
    public AudioClip sfxGotCoffee;
    public AudioClip sfxSleep;
    public AudioClip sfxUnicornLost;
    public AudioClip sfxUnicornExploded;
    public AudioClip sfxSuccess;
    public AudioClip sfx100Percent;
    public AudioClip sfxBlowUpUnicorns;

    AudioSource audioSource;
    public float playerCoffeeContent;

    public TextMeshProUGUI caffeineContentTextObject;
    public TextMeshProUGUI deadTextObject;
    public GameObject deadPanel;
    private WaterWell onTopOfWaterWell;
    public static Player Instance { get; private set; }
    public static bool Dead = false;

    /// <summary>
    /// Awake is called when the script instance is being loaded.
    /// </summary>
    void Awake()
    {
        Instance = this;
        spriteRenderer = GetComponent<SpriteRenderer>();
        audioSource = GetComponent<AudioSource>();
        Dead = false;
    }

    void Start()
    {
        Move.inputAction.PlayerActions.PickUpPutDown.started += _ => TryToPickup();
        Move.inputAction.PlayerActions.PickUpPutDown.canceled += _ => TryToPutdown();
        Move.inputAction.PlayerActions.Action.started += _ => DoAction();
    }

    private void Update()
    {
        if (Dead)
            return;

        playerCoffeeContent -= 0.25f * Time.deltaTime;
        if (playerCoffeeContent <= 0)
        {
            Dead = true;
            audioSource.PlayOneShot(sfxSleep);
            StartCoroutine(DeadScene());
        }
        else if (playerCoffeeContent >= 100f)
        {
            Dead = true;
            audioSource.PlayOneShot(sfxSuccess);
            StartCoroutine(WonScene());
            playerCoffeeContent = 100.00f;
        }

        UpdateCaffeineText();
    }
    private IEnumerator WonScene()
    {
        deadPanel.SetActive(true);
        deadTextObject.text = "You turned yourself into coffee!!\r\n.";
        yield return new WaitForSeconds(1.5f);
        audioSource.PlayOneShot(sfx100Percent);
        for (int i = 0; i < 40; i++)
        {
            yield return new WaitForSeconds(1f);
            deadTextObject.text += ".";
        }
        SceneManager.LoadScene(SceneManager.GetActiveScene().name);
    }

    private IEnumerator DeadScene()
    {
        deadPanel.SetActive(true);
        deadTextObject.text = "You fell asleep!\r\nForever!\r\n";
        for (int i = 0; i < 40; i++)
        {
            yield return new WaitForSeconds(1f);
            deadTextObject.text += ".";
        }
        SceneManager.LoadScene(SceneManager.GetActiveScene().name);
    }

    internal static void DeadByMissingUnicorn()
    {
        Player.Instance.DeathByMissingUnicorn();
    }

    private void DeathByMissingUnicorn()
    {
        Dead = true;
        audioSource.PlayOneShot(sfxUnicornLost);
        StartCoroutine(DeadByUnicornScene());
    }

    private IEnumerator DeadByUnicornScene()
    {
        deadPanel.SetActive(true);
        deadTextObject.text = "You .....!\r\nYou didn't close the gate!\r\nNow the Unicorn is gone!\r\n";
        for (int i = 0; i < 40; i++)
        {
            yield return new WaitForSeconds(1f);
            deadTextObject.text += ".";
        }
        SceneManager.LoadScene(SceneManager.GetActiveScene().name);
    }

    public void TryToPickup()
    {
        if (Dead)
            return;

        // Debug.Log("Trying to pick up somthing!");

        if (OnTopOf == null)
            return;

        if (holding != null)
        {
            // Debug.Log("You are already holding an object!");
            return;
        }

        OnTopOf.transform.position = transform.position + OnTopOfIPickable.GetOffset();
        OnTopOf.transform.parent = transform;
        holding = OnTopOf;
        OnTopOfIPickable.EnableCollider(false);
        spriteRenderer.sprite = sprLifting;
        audioSource.PlayOneShot(sfxPickup);
    }

    public void TryToPutdown()
    {
        if (Dead)
            return;

        if (holding == null)
        {
            // Debug.Log("You are not holding anything!");
            return;
        }

        holding.GetComponent<IPickupable>().EnableCollider(true);
        holding.transform.parent = null;
        holding.transform.position = dropPosition.position;
        holding = null;

        audioSource.PlayOneShot(sfxDrop);

        // Debug.Log("Trying to put something down!");
        CameraShake.Shake(0.1f, 0.1f, 2);
        spriteRenderer.sprite = sprNeutral;
    }

    public void DoAction()
    {
        if (Dead)
            return;

        if (onTopOfGate.state)
            onTopOfGate.gateObject.ToggleGate();

        if (coffeeSwitch.state)
            coffeeSwitch.switchObject.ToggleSwitch();

        if (onTopOfWaterWell != null)
            onTopOfWaterWell.SpawnWater();
    }

    private void OnTriggerEnter2D(Collider2D other)
    {
        IPickupable ip = other.GetComponent<IPickupable>();
        if (ip != null)
        {
            OnTopOf = other.gameObject;
            OnTopOfIPickable = ip;
            // Debug.Log("Enter: " + other.name);
        }

        if (other.CompareTag("Gate"))
            onTopOfGate = new Gate.GateInfo() { state = true, gateObject = other.GetComponent<Gate>() };
        if (other.CompareTag("WaterWell"))
            onTopOfWaterWell = other.GetComponent<WaterWell>();

        if (other.CompareTag("Switch"))
            coffeeSwitch = new CoffeeMakerSwitch.CoffeeMakerSwitchInfo()
            {
                state = true,
                switchObject = other.GetComponent<CoffeeMakerSwitch>()
            };
        if (other.CompareTag("Coffee"))
        {
            Destroy(other.gameObject);
            audioSource.PlayOneShot(sfxGotCoffee);
            playerCoffeeContent += 10f;
            UpdateCaffeineText();
        }
    }

    private void UpdateCaffeineText()
    {
        caffeineContentTextObject.SetText("Phils Coffee content:" + playerCoffeeContent.ToString("00.00") + "%");
    }

    private void OnTriggerExit2D(Collider2D other)
    {
        IPickupable ip = other.GetComponent<IPickupable>();
        if (ip != null)
        {
            OnTopOf = null;
            OnTopOfIPickable = null;
        }

        if (other.CompareTag("Gate"))
            onTopOfGate = new Gate.GateInfo() { state = false, gateObject = null };

        if (other.CompareTag("WaterWell"))
            onTopOfWaterWell = null;

        if (other.CompareTag("Switch"))
            coffeeSwitch = new CoffeeMakerSwitch.CoffeeMakerSwitchInfo() { state = false, switchObject = null };
    }

    internal static void DeadBygUnicornExploding()
    {
        Player.Instance.DeathBygUnicornExploding();
    }

    private void DeathBygUnicornExploding()
    {
        Dead = true;
        audioSource.PlayOneShot(sfxUnicornExploded);
        StartCoroutine(DeadByUnicornSceneExploding());
    }

    private IEnumerator DeadByUnicornSceneExploding()
    {
        yield return new WaitForSeconds(1f);
        deadPanel.SetActive(true);
        deadTextObject.text = "So... You blew up the unicorn!\r\nSigh.";
        yield return new WaitForSeconds(1f);
        audioSource.PlayOneShot(sfxBlowUpUnicorns);
        for (int i = 0; i < 40; i++)
        {
            yield return new WaitForSeconds(1f);
            deadTextObject.text += ".";
        }
        SceneManager.LoadScene(SceneManager.GetActiveScene().name);
    }
}
