﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using TMPro;

public class Unicorn : MonoBehaviour
{
    public Player player;
    public Gate theGate;

    int currentNeedIndex = 0;

    string[] possibleNeeds = new string[] { "Water", "Flower", "Hay" };

    public AudioClip[] sfxPoops;
    public AudioClip sfxWrongFood;
    AudioSource audioSource;
    public Transform poopPosition;
    public GameObject coffeeBeanPrefab;

    public TextMeshProUGUI needsTextObject;

    private Animator animator;
    public ParticleSystem explosion;
    public Gradient uniColors;

    public AudioClip sfxUniNeeds;
    public AudioClip[] sfxItems;
    void Start()
    {
        currentNeedIndex = Random.Range(0, possibleNeeds.Length);
        audioSource = GetComponent<AudioSource>();
        animator = GetComponent<Animator>();

        UpdateNeedsText();
        StartCoroutine(UniNeeds());
    }

    void Update()
    {
        if (Player.Dead)
            return;
        if (theGate.GetState() && (transform.position - player.transform.position).magnitude > 8f)
        {
            Player.DeadByMissingUnicorn();
        }
    }

    private IEnumerator UniNeeds()
    {
        audioSource.PlayOneShot(sfxUniNeeds);
        yield return new WaitForSeconds(sfxUniNeeds.length);
        audioSource.PlayOneShot(sfxItems[currentNeedIndex]);
    }

    int frame = 0;
    void OnTriggerEnter2D(Collider2D other)
    {
        if (other.CompareTag(possibleNeeds[currentNeedIndex]))
        {
            if (frame == Time.frameCount) // Unicorn has 2 colliders and both triggers an event.!
                return;
            frame = Time.frameCount;
            Destroy(other.gameObject);
            currentNeedIndex = Random.Range(0, possibleNeeds.Length);
            StartCoroutine(UniNeeds());
            UpdateNeedsText();
            animator.SetBool("Poop", true);
        }
        if (!other.CompareTag("Bean") && !other.CompareTag("Player"))
        {
            //Wrong food. Explode
            if (frame == Time.frameCount) // Unicorn has 2 colliders and both triggers an event.!
                return;
            Destroy(other.gameObject);
            audioSource.PlayOneShot(sfxWrongFood);
            animator.SetBool("Explode", true);
        }
    }

    public void UpdateNeedsText()
    {
        needsTextObject.SetText("Unicorn wants: " + possibleNeeds[currentNeedIndex]);
    }

    public void DeliverBean()
    {
        Instantiate(coffeeBeanPrefab, poopPosition.position, new Quaternion(), null);
        audioSource.PlayOneShot(sfxPoops[Random.Range(0, sfxPoops.Length)]);
        animator.SetBool("Poop", false);
    }


    ParticleSystem.Particle[] m_Particles = null;
    public void UnicornExploded()
    {
        Player.DeadBygUnicornExploding();

        m_Particles = new ParticleSystem.Particle[explosion.main.maxParticles];
        explosion.Emit(100);

        int numParticlesAlive = explosion.GetParticles(m_Particles);

        // Change only the particles that are alive
        for (int i = 0; i < numParticlesAlive; i++)
        {
            m_Particles[i].color = uniColors.Evaluate(Random.value);
        }

        GetComponent<SpriteRenderer>().enabled = false;
    }
}
